﻿using ClickHouse.Ado;
using Serilog;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;

namespace BenchmarkTool
{
    public class ClickhouseDB : IDatabase
    {
        private ClickHouseConnection connection;
        
        public void cleanup()
        {
            throw new NotImplementedException();
        }

        public void close()
        {
            try
            {
                if (connection != null)
                    connection.Close();
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to close Clickhouse. Exception: {0}", ex.ToString()));
            }
        }

        public void init()
        {
            try
            {
                var settings = new ClickHouseConnectionSettings(String.Format("Host={0};Port={1};Database={2};User={3}", Config.GetClickhouseHost(), Config.GetClickhousePort(), Config.GetClickhouseDatabase(), Config.GetClickhouseUser() ));
                connection = new ClickHouseConnection(settings);
                connection.Open();
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to initialize Clickhouse. Exception: {0}", ex.ToString()));
            }
        }

        public QueryStatus OutOfRangeQuery(OORangeQuery query)
        {
            int points = 0;
            try
            {
                string sql = Query.ClickhouseOutOfRangeQuery;
                sql = sql.Replace(QueryParams.StartParam, query.StartDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.EndParam, query.EndDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.SensorIDParam, query.SensorID.ToString());
                sql = sql.Replace(QueryParams.MinValParam, query.MinValue.ToString());
                sql = sql.Replace(QueryParams.MaxValParam, query.MaxValue.ToString());

                Log.Information(sql);
                var cmd = connection.CreateCommand();
                cmd.CommandText = sql;
                var reader = cmd.ExecuteReader();
                reader.ReadAll(rowReader => { points++; });
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Out of Range Query on ClickhouseDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        public QueryStatus RangeQueryAgg(RangeQuery rangeQuery)
        {
            int points = 0;
            try
            {
                string sql = Query.ClickhouseRangeQueryAgg;
                sql = sql.Replace(QueryParams.StartParam, rangeQuery.StartDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.EndParam, rangeQuery.EndDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.SensorIDsParam, rangeQuery.SensorFilter);
                
                Log.Information(sql);
                var cmd = connection.CreateCommand();
                cmd.CommandText = sql;
                var reader = cmd.ExecuteReader();
                reader.ReadAll(rowReader => { points++; });
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Range Query Agg Data on ClickhouseDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        public QueryStatus RangeQueryRaw(RangeQuery rangeQuery)
        {
            int points = 0;
            try
            {
                string sql = Query.ClickhouseRangeQueryRaw;
                sql = sql.Replace(QueryParams.StartParam, rangeQuery.StartDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.EndParam, rangeQuery.EndDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.SensorIDsParam, rangeQuery.SensorFilter);
                Log.Information(sql);
                var cmd = connection.CreateCommand();
                cmd.CommandText = sql;
                var reader = cmd.ExecuteReader();
                reader.ReadAll(rowReader => { points++; });
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Range Query Raw Data on ClickhouseDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        public QueryStatus AggregatedDifferenceQuery(ComparisionQuery query)
        {
            int points = 0;
            try
            {
                string sql = Query.ClickhouseAggDifferenceQuery;
                sql = sql.Replace(QueryParams.StartParam, query.StartDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.EndParam, query.EndDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.FirstSensorIDParam, query.FirstSensorID.ToString());
                sql = sql.Replace(QueryParams.SecondSensorIDParam, query.SecondSensorID.ToString());

                Log.Information(sql);
                var cmd = connection.CreateCommand();
                cmd.CommandText = sql;
                var reader = cmd.ExecuteReader();
                reader.ReadAll(rowReader => { points++; });
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Difference beween agg sensor values query on ClickhouseDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }



        //public QueryStatus WriteBatch(Batch batch)
        //{
        //    try
        //    {
        //        var command = connection.CreateCommand();
        //        command.CommandText = "INSERT INTO sensor_data (sensor_id, value, time) VALUES @bulk";
        //        command.Parameters.Add(new ClickHouseParameter
        //        {
        //            ParameterName = "bulk",
        //            Value = batch.Records
        //        });
        //        command.ExecuteNonQuery(); 
        //        return new QueryStatus(true);
        //    }
        //    catch (Exception ex)
        //    {
        //        Log.Error(String.Format("Failed to insert batch into Cickhouse. Exception: {0}", ex.ToString()));
        //        return new QueryStatus(false, 0, ex, ex.ToString());
        //    }
        //}

        public QueryStatus StandardDevQuery(SpecificQuery query)
        {
            int points = 0;
            try
            {
                string sql = Query.ClickhouseSTDEVQuery;
                sql = sql.Replace(QueryParams.StartParam, query.StartDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.EndParam, query.EndDate.ToString("yyyy-MM-dd HH:mm:ss"));
                sql = sql.Replace(QueryParams.SensorIDParam, query.SensorID.ToString());
                Log.Information(sql);
                var cmd = connection.CreateCommand();
                cmd.CommandText = sql;
                var reader = cmd.ExecuteReader();
                reader.ReadAll(rowReader => { points++; });
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute STD Dev query on ClickhouseDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        async Task<QueryStatus> IDatabase.WriteBatch(Batch batch)
        {
            try
            {
                var command = connection.CreateCommand();
                
                command.CommandText = String.Format("INSERT INTO {0} ({1}, {2}, {3}) VALUES @bulk", Constants.DatabaseName, Constants.SensorID, Constants.Value, Constants.Time);
                command.Parameters.Add(new ClickHouseParameter
                {
                    ParameterName = "bulk",
                    Value = batch.Records
                });
                command.ExecuteNonQuery();
                return new QueryStatus(true);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to insert batch into Cickhouse. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }
    }
}
