﻿using Serilog;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Threading.Tasks;

namespace BenchmarkTool
{
    public class ClientRead
    {
        public PerformanceMetricRead PerformanceMetric { get; private set; }
        private DatabaseWrapper wrapper;
        private int ClientsNumber;
        private DateTime Date;
        

      
        public ClientRead(int clientsNumber, DateTime date ) 
        {
            Date = date;
            ClientsNumber = clientsNumber;
            PerformanceMetric = new PerformanceMetricRead(Config.GetOperation(), Date, Config.GetDurationMinutes(), int.Parse(Config.GetAggregationInterval()));
            wrapper = new DatabaseWrapper(PerformanceMetric);
            wrapper.init();

        }
        public async Task RunQuery()
        {
            var operation = Config.GetOperation();
            int loop = Config.GetLoop();
            for (var i = 0; i < loop; i++)
            {
                switch (operation.ToEnum<Operation>())
                {
                   
                    case Operation.RangeQueryRawData:
                        Log.Information("Executing Range Raw Query");
                        RangeQuery query = new RangeQuery(Date, Config.GetDurationMinutes(), Config.GetSensorsFilter(), Config.GetSensorsFilterString());
                        wrapper.RangeQueryRaw(query);
                        break;
                    case Operation.RangeQueryAggData:
                        Log.Information("Executing Range Aggregared Query");
                        RangeQuery aggQuery = new RangeQuery(Date, Config.GetDurationMinutes(), Config.GetSensorsFilter(), Config.GetSensorsFilterString());
                        wrapper.RangeQueryAgg(aggQuery);
                        break;
                    case Operation.OutOfRangeQuery:
                        Log.Information("Executing Out of Range Query");
                        OORangeQuery oorangeQuery = new OORangeQuery(Date, Config.GetDurationMinutes(), Config.GetSensorID(), Config.GetMaxValue(), Config.GetMinValue());
                        wrapper.OutOfRangeQuery(oorangeQuery);
                        break;
                    case Operation.DifferenceAggQuery:
                        Log.Information("Executing Agg Difference Query");
                        ComparisionQuery comparisonQuery = new ComparisionQuery(Date, Config.GetDurationMinutes(), Config.GetFirstSensorID(), Config.GetSecondSensorID());
                        wrapper.AggregatedDifferenceQuery(comparisonQuery);
                        break;
                    case Operation.STDDevQuery:
                        Log.Information("Executing Standard Deviation Query");
                        wrapper.StandardDevQuery(new SpecificQuery(Date, Config.GetDurationMinutes(), Config.GetSecondSensorID()));
                        break;
                    default:
                        throw new NotImplementedException();
                }
            }
            wrapper.close();
            //Log.Information(String.Format("Client {0} completed", Index.ToString()));
        }
    }
}
