﻿using Serilog;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Threading.Tasks;

namespace BenchmarkTool
{
    public class Client
    {
        public PerformanceMetricWrite PerformanceMetric { get; set; }
        private DatabaseWrapper wrapper;
        public int Index;
        private int ClientsNumber;
        private int BatchSize;
        private int SensorNumber;
        private DateTime Date;
        private int GlobalIndex;

        public Client(int index, int globalIndex = 0)
        {
            try
            {
                Index = index;
                PerformanceMetric = new PerformanceMetricWrite();
                PerformanceMetric.PerformedOperation = Config.GetOperation();
                GlobalIndex = globalIndex;
                wrapper = new DatabaseWrapper(PerformanceMetric);
                wrapper.init();

            }
            catch (Exception ex)
            {
                Log.Error(ex.ToString());
            }
        }


        public Client(int index, int clientsNumber, int batchSize, int sensorNumber, DateTime date, int globalIndex = 0) : this(index, globalIndex)
        {
            ClientsNumber = clientsNumber;
            BatchSize = batchSize;
            SensorNumber = sensorNumber;
            Date = date;
            PerformanceMetric.BatchSize = batchSize;
            PerformanceMetric.ClientsNumber = clientsNumber;
            PerformanceMetric.SensorsNumber = sensorNumber;


        }

        public async Task RunQuery()
        {
            var operation = Config.GetOperation();
            int loop = Config.GetLoop();
            //var batchSize = Config.GetBatchSize();
            //int clientsNumber = Config.GetClientNumber();
            //int sensorNumber = Config.GetSensorNumber();
            DataGenerator dataGenerator = new DataGenerator();
            decimal sensorsPerClient = SensorNumber > ClientsNumber ? Math.Ceiling(Convert.ToDecimal(SensorNumber / ClientsNumber)) : SensorNumber;
            int startId = SensorNumber > ClientsNumber ? Convert.ToInt32(Index * sensorsPerClient) : 0;
            for (var i = 0; i < loop; i++)
            {



                var batch = dataGenerator.GenerateBatch(BatchSize, startId, sensorsPerClient, i, GlobalIndex, Date);
                await wrapper.WriteBatch(batch);


            }
            wrapper.close();
            //Log.Information(String.Format("Client {0} completed", Index.ToString()));
        }
    }
}

