﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;

namespace BenchmarkTool
{
    public class DataGenerator : IDataGenerator
    {

        public Batch GenerateBatch(int batchSize, int sensorStartId, decimal sensorsPerClient, int loopIndex, int globalIndex)
        {
            int loop = Config.GetLoop();
            RecordFactory recordFactory = new RecordFactory();
            Random rndval = new Random();
            Batch batch = new Batch(batchSize);
            var start = (loop * globalIndex + loopIndex) * batchSize + sensorStartId >= sensorsPerClient + sensorStartId ? sensorStartId : (loop * globalIndex + loopIndex) * batchSize + sensorStartId;
            var timestamp = GetTimestamp(loopIndex, globalIndex);
            for (int k = 0; k < batchSize; k++)
            {
                batch.Records.Add(recordFactory.Create(start, timestamp.AddMilliseconds(5), rndval.Next()));
                start++;
                if (start >= sensorStartId + sensorsPerClient)
                    start = sensorStartId;
            }
            return batch;
        }

        public Batch GenerateBatch(int batchSize, int sensorStartId, decimal sensorsPerClient, int loopIndex, int globalIndex, DateTime date)
        {
            int loop = Config.GetLoop();
            RecordFactory recordFactory = new RecordFactory();
            Random rndval = new Random();
            Batch batch = new Batch(batchSize);
            var start = (loop * globalIndex + loopIndex) * batchSize + sensorStartId >= sensorsPerClient + sensorStartId ? sensorStartId : (loop * globalIndex + loopIndex) * batchSize + sensorStartId;
            var timestamp = GetTimestamp(date, loopIndex, globalIndex);
            for (int k = 0; k < batchSize; k++)
            {
                batch.Records.Add(recordFactory.Create(start, timestamp, rndval.Next()));
                start++;
                if (start >= sensorStartId + sensorsPerClient)
                    start = sensorStartId;
            }
            return batch;
        }

        private static DateTime GetTimestamp(int loopIndex, int globalIndex)
        {
            var date = Config.GetStartTime();
            return date.AddMilliseconds(Config.GetBatchTimeOffset() * loopIndex + Config.GetGlobalTimeOffset() * globalIndex * 60000);
        }

        private static DateTime GetTimestamp(DateTime date,int loopIndex, int globalIndex)
        {
            //var date = Config.GetStartTime();
            return date.AddMilliseconds(Config.GetBatchTimeOffset() * loopIndex + Config.GetGlobalTimeOffset() * globalIndex * 1000);
        }
    }
}
