﻿using CsvHelper;
using Serilog;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace BenchmarkTool
{
    public class PerformanceMetricRead : PerformanceMetricBase, IMetric
    {
        public DateTime StartDate { get; set; }
        public long Duration { get; set; }
        string filter { get; set; }
        int Aggregation { get; set; }

        public PerformanceMetricRead(string operation, DateTime date, long duration, int aggregation)
        {
            PerformedOperation = operation;
            Aggregation = aggregation;
            Duration = duration;
            StartDate = date;
        }
        public override void WriteToCSV()
        {
            try
            {
                Log.Information(String.Format("Start Date: {0}", StartDate.ToString()));
                var path = Config.GetMetricsCSVPath();
                bool exists = File.Exists(path);
                using (StreamWriter sw = new StreamWriter(path, true, new UTF8Encoding(true)))
                {
                    using (CsvWriter cw = new CsvWriter(sw, new CsvHelper.Configuration.CsvConfiguration(System.Globalization.CultureInfo.CurrentCulture)))
                    {
                        if (!exists)
                        {
                            cw.WriteHeader<LogRecordRead>();
                            cw.NextRecord();
                        }

                        var latency = ExecutionTime;

                        if (latency > 0 && (SucceededOperationDataPointsNumber > 0 || FailedOperationDataPointsNumber > 0))
                        {
                            var record = new LogRecordRead(latency, SucceededOperationDataPointsNumber, FailedOperationDataPointsNumber, SucceededOperationsNumber, FailedOperationsNumber, PerformedOperation, ClientsNumber, StartDate, Duration, Aggregation);
                            cw.WriteRecord<LogRecordRead>(record);
                            cw.NextRecord();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Log.Error(ex.ToString());
            }
        }
    }
}
