﻿using CsvHelper;
using Serilog;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace BenchmarkTool
{
    public class PerformanceMetricWrite : PerformanceMetricBase, IMetric
    {
        public int BatchSize { get; set; }

        public int SensorsNumber { get; set; }

        public PerformanceMetricWrite() { }
        public PerformanceMetricWrite(int batchSize, int clientNb, int sensorNb)
        {
            BatchSize = batchSize;
            ClientsNumber = clientNb;
            SensorsNumber = sensorNb;
        }
        public override void WriteToCSV()
        {
            try
            {
                var path = Config.GetMetricsCSVPath();
                bool exists = File.Exists(path);
                using (StreamWriter sw = new StreamWriter(path, true, new UTF8Encoding(true)))
                {
                    using (CsvWriter cw = new CsvWriter(sw, new CsvHelper.Configuration.CsvConfiguration(System.Globalization.CultureInfo.CurrentCulture)))
                    {
                        if (!exists)
                        {
                            cw.WriteHeader<LogRecordWrite>();
                            cw.NextRecord();
                        }

                        var latency = ExecutionTime;

                            if (latency > 0 && (SucceededOperationDataPointsNumber > 0 || FailedOperationDataPointsNumber> 0))
                            {
                                var record = new LogRecordWrite(latency, SucceededOperationDataPointsNumber, FailedOperationDataPointsNumber, SucceededOperationsNumber, FailedOperationsNumber, PerformedOperation, ClientsNumber, BatchSize, SensorsNumber);
                                cw.WriteRecord<LogRecordWrite>(record);
                                cw.NextRecord();
                            }
                        
                    }
                }
            }
            catch (Exception ex)
            {
                Log.Error(ex.ToString());
            }
        }

     
    }
}
