﻿using System;
using System.Collections.Generic;
using System.Text;

namespace BenchmarkTool
{
    public static class Query
    {
        private static string _timescaleRangeQueryAgg = @"SELECT time_bucket('{0}m', {1}) AS time_agg, sensor_id,  avg({2}) FROM {3}
                                                        where {1} >={4} and {1}<={5} and {6} = ANY({7}) group by time_agg, sensor_id";

        private static string _timescaleRangeQueryRaw = @"SELECT * FROM {0} where {1} >={2} and {1}<={3} and {4} = ANY({5})";


        private static string _timescaleOutOfRangeQuery = @"SELECT time_bucket('{0}m', {1}) AS time_agg , max({2}), min({2}) FROM {3} 
                                                            where {1} >={4} and {1}<={5} and {6} = {7} 
                                                            group by time_agg having min({2}) < {8} OR max({2}) >{9}";

        public static string _timescaleSTDEVQuery = @"SELECT stddev({0}) FROM {1} where {2} >={3} and {2}<={4} and {5} = {6}";


        private static string _fluxRangeQueryRaw = @"from(bucket: ""{0}"")   
                                                        |> range(start: {1}, stop: {2})   
                                                        |> filter(fn: (r) => r[""_measurement""] == ""{3}"")   
                                                        |> filter(fn: (r) => r[""{4}""] =~ {5}) ";

        private static string _fluxRangeQueryAgg = @"from(bucket: ""{0}"")   
                                                        |> range(start: {1}, stop: {2})   
                                                        |> filter(fn: (r) => r[""_measurement""] == ""{3}"")   
                                                        |> filter(fn: (r) => r[""{4}""] =~ {5})   
                                                        |> aggregateWindow(every: {6}m, fn: mean, createEmpty: false)  
                                                        |> yield(name: ""mean"")";


        private static string _fluxOutOfRangeQuery = @"data = from(bucket: ""{0}"")  
                                                        |> range(start: {1}, stop: {2})     
                                                        |> filter(fn: (r) => r[""_measurement""] == ""{3}"")   
                                                        |> filter(fn: (r) => r[""{4}""] == ""{5}"") 
                                                        min = data   
                                                            |> aggregateWindow(   column: ""_{6}"", every: {7}m, fn: min   ) 
                                                        max = data   
                                                            |> aggregateWindow( column: ""_{6}"", every: {7}m, fn: max  ) 
                                                        join(tables: {{min: min, max: max}}, on: [""_start"", ""_stop"", ""_time"", ""_field"", ""_measurement"", ""{8}""], method: ""inner"") 
                                                            |> filter(fn: (r) => r[""_{6}_max""] > {9} or r[""_{6}_min""]  < {10})";

        private static string _fluxSTDEVQuery = @"from(bucket: ""{0}"")   
                                                        |> range(start: {1}, stop: {2})   
                                                        |> filter(fn: (r) => r[""_measurement""] == ""{3}"" and r[""{4}""] == ""{5}"")   
                                                        |> stddev()";

        private static string _fluxAggDifferenceQuery = @"data = from(bucket: ""{0}"")
                                                        |> range(start: {1}, stop: {2})
                                                        |> filter(fn: (r) => r[""_measurement""] == ""{3}"")   
                                                        |> filter(fn: (r) => r[""{4}""] == ""{5}"" or r[""{4}""] == ""{6}"") 
                                                        sen1 = data   
                                                            |> filter(fn: (r) => r[""{4}""] == ""{5}"")
                                                            |> aggregateWindow(column: ""_{7}"", every: {8}m, fn: mean , createEmpty: false  )
                                                        sen2 = data   
                                                            |> filter(fn: (r) => r[""{4}""] == ""{6}"")
                                                            |> aggregateWindow(column: ""_{7}"", every: {8}m, fn: mean , createEmpty: false  )
                                                        join(tables: {{ sen1: sen1, sen2: sen2}}, on: [""_start"", ""_stop"", ""_time"", ""_field"", ""_measurement""], method: ""inner"") 
                                                            |> map(fn: (r) =>({{
                                                                _time: r._time,
                                                                _value: r._{7}_sen2 - r._{7}_sen1 }}))";

        private static string _clickhouseRangeQueryRaw = @"SELECT * from {0} 
                                                            where {1} >=toDateTime('{2}') and {1}<=toDateTime('{3}') and {4} IN ({5})";


        private static string _clickhouseRangeQueryAgg = @"SELECT toStartOfInterval({0} ,INTERVAL {1} minute) as interval, avg({2}), {3} from {4} 
                                                            where {0} >=toDateTime('{5}') and {0}<=toDateTime('{6}') and {3} IN ({7})   
                                                            GROUP BY {3}, interval";

        private static string _clickhouseOutOfRangeQuery = @"SELECT toStartOfInterval({0} ,INTERVAL {1} minute) as interval,max({2}), min({2}), {3} from {4} 
                                                             where {0} >=toDateTime('{5}') and {0}<=toDateTime('{6}') and {3} = {7} 
                                                             GROUP BY {3}, interval 
                                                             HAVING max({2}) > {8} or  min({2}) < {9}";


        private static string _timescaleAggDifferenceQuery = @"select A1.time_agg, A2.val - A1.val as difference from  
                                                                (SELECT time_bucket('{0}m', {1}) AS time_agg , avg({2}) as val FROM {3} 
                                                                    where {1} >={4} and {1}<={5} and {6} = {7} group by time_agg)A1 
                                                                inner join 
                                                                (SELECT time_bucket('{0}m', {1}) AS time_agg , avg({2}) as val FROM {3} 
                                                                    where {1} >={4} and {1}<={5} and {6} = {8}  group by time_agg)A2 
                                                                On A1.time_agg = A2.time_agg";


        public static string _clickhouseAggDifferenceQuery = @"SELECT A1.interval, A2.val - A1.val from
                                                                (SELECT toStartOfInterval({0} ,INTERVAL {1} minute) as interval, avg({2}) as val from {3} 
                                                                    where {0} >=toDateTime('{4}') and {0}<=toDateTime('{5}') and {6} = {7}
                                                                    GROUP BY interval)A1
                                                                INNER JOIN 
                                                                (SELECT toStartOfInterval({0} ,INTERVAL {1} minute) as interval, avg({2}) as val from {3} 
                                                                    where {0} >=toDateTime('{4}') and {0}<=toDateTime('{5}') and {6} = {8}
                                                                    GROUP BY interval)A2
                                                                ON A1.interval = A2.interval";

        private static string _clickhouseSTDEVQuery = @"SELECT stddevSamp({0}) from {1} 
                                                            where {2} >=toDateTime('{3}') and {2}<=toDateTime('{4}') and {5} = {6}";

        public static string TimescaleRangeQueryAgg = String.Format(_timescaleRangeQueryAgg, Config.GetAggregationInterval(), Constants.Time, Constants.Value, Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDsParam);

        public static string TimescaleRangeQueryRaw = String.Format(_timescaleRangeQueryRaw, Constants.DatabaseName, Constants.Time, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDsParam);

        public static string TimescaleOutOfRangeQuery = String.Format(_timescaleOutOfRangeQuery, Config.GetAggregationInterval(), Constants.Time, Constants.Value, Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDParam, QueryParams.MinValParam, QueryParams.MaxValParam);

        public static string TimescaleAggDifferenceQuery = String.Format(_timescaleAggDifferenceQuery, Config.GetAggregationInterval(), Constants.Time, Constants.Value, Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.FirstSensorIDParam, QueryParams.SecondSensorIDParam);

        public static string TimescaleSTDEVQuery = String.Format(_timescaleSTDEVQuery, Constants.Value, Constants.DatabaseName, Constants.Time, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDParam);

        public static string FluxRangeQueryRaw = String.Format(_fluxRangeQueryRaw, Config.GetInfluxBucket(), QueryParams.StartParam, QueryParams.EndParam, Constants.DatabaseName, Constants.SensorID, QueryParams.SensorIDsParam);

        public static string FluxSTDEVQuery = String.Format(_fluxSTDEVQuery, Config.GetInfluxBucket(), QueryParams.StartParam, QueryParams.EndParam, Constants.DatabaseName, Constants.SensorID, QueryParams.SensorIDParam);

        public static string FluxRangeQueryAgg = String.Format(_fluxRangeQueryAgg, Config.GetInfluxBucket(), QueryParams.StartParam, QueryParams.EndParam, Constants.DatabaseName, Constants.SensorID, QueryParams.SensorIDsParam, Config.GetAggregationInterval());

        public static string FluxOutOfRangeQuery = String.Format(_fluxOutOfRangeQuery, Config.GetInfluxBucket(), QueryParams.StartParam, QueryParams.EndParam, Constants.DatabaseName, Constants.SensorID, QueryParams.SensorIDParam, Constants.Value, Config.GetAggregationInterval(), Constants.SensorID, QueryParams.MinValParam, QueryParams.MaxValParam);

        public static string FluxAggDifferenceQuery = String.Format(_fluxAggDifferenceQuery, Config.GetInfluxBucket(), QueryParams.StartParam, QueryParams.EndParam,
                                                                    Constants.DatabaseName, Constants.SensorID, QueryParams.FirstSensorIDParam, QueryParams.SecondSensorIDParam, Constants.Value, Config.GetAggregationInterval());

        public static string ClickhouseRangeQueryRaw = String.Format(_clickhouseRangeQueryRaw, Constants.DatabaseName, Constants.Time, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDsParam);

        public static string ClickhouseRangeQueryAgg = String.Format(_clickhouseRangeQueryAgg, Constants.Time, Config.GetAggregationInterval(), Constants.Value, Constants.SensorID, Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, QueryParams.SensorIDsParam);

        public static string ClickhouseOutOfRangeQuery = String.Format(_clickhouseOutOfRangeQuery, Constants.Time, Config.GetAggregationInterval(), Constants.Value, Constants.SensorID, Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, QueryParams.SensorIDParam, QueryParams.MinValParam, QueryParams.MaxValParam);

        public static string ClickhouseAggDifferenceQuery = String.Format(_clickhouseAggDifferenceQuery, Constants.Time, Config.GetAggregationInterval(), Constants.Value,
            Constants.DatabaseName, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.FirstSensorIDParam, QueryParams.SecondSensorIDParam);


        public static string ClickhouseSTDEVQuery = String.Format(_clickhouseSTDEVQuery, Constants.Value, Constants.DatabaseName, Constants.Time, QueryParams.StartParam, QueryParams.EndParam, Constants.SensorID, QueryParams.SensorIDParam);

    }
}
