﻿using Npgsql;
using PostgreSQLCopyHelper;
using Serilog;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Threading.Tasks;

namespace BenchmarkTool
{
    public class TimescaleDB : IDatabase
    {
        private NpgsqlConnection connection;
        PostgreSQLCopyHelper<IRecord> copyHelper;

       

        public void cleanup()
        {
            throw new NotImplementedException();
        }

        public void close()
        {
            try
            {
                if (connection != null)
                    connection.Close();
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to close TimescaleDB. Exception: {0}", ex.ToString()));
            }
        }

        public void init()
        {
            try
            {  
                connection = new NpgsqlConnection(Config.GetTimescaleConnection());
                copyHelper = new PostgreSQLCopyHelper<IRecord>(Constants.SchemaName, Constants.DatabaseName)
                       .MapTimeStamp(Constants.Time, x => x.Time)
                       .MapInteger(Constants.SensorID, x => x.SensorID)
                       .MapDouble(Constants.Value, x => x.Value);
                connection.Open();
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to initialize TimescaleDB. Exception: {0}", ex.ToString()));
            }
        }

        public QueryStatus OutOfRangeQuery(OORangeQuery query)
        {
            try
            {
                string sql = Query.TimescaleOutOfRangeQuery;        
                Log.Information(sql);
                using var cmd = new NpgsqlCommand(sql, connection);
                cmd.Parameters.AddWithValue(QueryParams.Start, NpgsqlTypes.NpgsqlDbType.Timestamp, query.StartDate);
                cmd.Parameters.AddWithValue(QueryParams.End, NpgsqlTypes.NpgsqlDbType.Timestamp, query.EndDate);
                cmd.Parameters.AddWithValue(QueryParams.SensorID, NpgsqlTypes.NpgsqlDbType.Integer, query.SensorID);
                cmd.Parameters.AddWithValue(QueryParams.MaxVal, NpgsqlTypes.NpgsqlDbType.Double, query.MaxValue);
                cmd.Parameters.AddWithValue(QueryParams.MinVal, NpgsqlTypes.NpgsqlDbType.Double, query.MinValue);

                using NpgsqlDataReader reader = cmd.ExecuteReader();
                var points = 0;
                while (reader.Read())
                {
                    points++;
                }
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Out of Range Query on TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }


        public QueryStatus AggregatedDifferenceQuery(ComparisionQuery query)
        {
            try
            {
                string sql = Query.TimescaleAggDifferenceQuery;
                Log.Information(sql);
                
                using var cmd = new NpgsqlCommand(sql, connection);
                cmd.Parameters.AddWithValue(QueryParams.Start, NpgsqlTypes.NpgsqlDbType.Timestamp, query.StartDate);
                cmd.Parameters.AddWithValue(QueryParams.End, NpgsqlTypes.NpgsqlDbType.Timestamp, query.EndDate);
                cmd.Parameters.AddWithValue(QueryParams.FirstSensorID, NpgsqlTypes.NpgsqlDbType.Integer, query.FirstSensorID);
                cmd.Parameters.AddWithValue(QueryParams.SecondSensorID, NpgsqlTypes.NpgsqlDbType.Integer, query.SecondSensorID);

                using NpgsqlDataReader reader = cmd.ExecuteReader();
                var points = 0;
                while (reader.Read())
                {
                    points++;
                }
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Difference beween agg sensor values query on TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }


        public QueryStatus RangeQueryAgg(RangeQuery rangeQuery)
        {
            try
            {
                string sql = Query.TimescaleRangeQueryAgg;
                Log.Information(sql);
                Log.Information("Start date: {0}, end date: {1}, sensors: {2}", rangeQuery.StartDate, rangeQuery.EndDate, rangeQuery.SensorFilter);
                using var cmd = new NpgsqlCommand(sql, connection);
                cmd.Parameters.AddWithValue(QueryParams.Start, NpgsqlTypes.NpgsqlDbType.Timestamp, rangeQuery.StartDate);
                cmd.Parameters.AddWithValue(QueryParams.End, NpgsqlTypes.NpgsqlDbType.Timestamp, rangeQuery.EndDate);
                cmd.Parameters.AddWithValue(QueryParams.SensorIDs, NpgsqlTypes.NpgsqlDbType.Array | NpgsqlTypes.NpgsqlDbType.Integer, rangeQuery.SensorIDs);

                using NpgsqlDataReader reader = cmd.ExecuteReader();
                var points = 0;
                while (reader.Read())
                {
                    points++;
                }
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Range Query Aggregated Data on TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        public QueryStatus RangeQueryRaw(RangeQuery rangeQuery)
        {
            try
            {
                string sql = Query.TimescaleRangeQueryRaw;
                Log.Information(sql);
                Log.Information(String.Format("Start Date: {0}", rangeQuery.StartDate.ToString()));
                Log.Information(String.Format("End Date: {0}", rangeQuery.EndDate.ToString()));
                using var cmd = new NpgsqlCommand(sql, connection);
                cmd.Parameters.AddWithValue(QueryParams.Start, NpgsqlTypes.NpgsqlDbType.Timestamp, rangeQuery.StartDate);
                cmd.Parameters.AddWithValue(QueryParams.End, NpgsqlTypes.NpgsqlDbType.Timestamp, rangeQuery.EndDate);
                cmd.Parameters.AddWithValue(QueryParams.SensorIDs, NpgsqlTypes.NpgsqlDbType.Array | NpgsqlTypes.NpgsqlDbType.Integer, rangeQuery.SensorIDs);
                using NpgsqlDataReader reader = cmd.ExecuteReader();
                var points = 0;
                while (reader.Read())
                {
                    points++;
                }
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute Range Query Raw Data on TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        public QueryStatus StandardDevQuery(SpecificQuery query)
        {
            try
            {
                string sql = Query.TimescaleSTDEVQuery;
                
                using var cmd = new NpgsqlCommand(sql, connection);
                cmd.Parameters.AddWithValue(QueryParams.Start, NpgsqlTypes.NpgsqlDbType.Timestamp, query.StartDate);
                cmd.Parameters.AddWithValue(QueryParams.End, NpgsqlTypes.NpgsqlDbType.Timestamp, query.EndDate);
                cmd.Parameters.AddWithValue(QueryParams.SensorID, NpgsqlTypes.NpgsqlDbType.Integer, query.SensorID);
                using NpgsqlDataReader reader = cmd.ExecuteReader();
                var points = 0;
                while (reader.Read())
                {
                    points++;
                }
                return new QueryStatus(true, points);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to execute STD Dev query on TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

        //public QueryStatus WriteBatch(Batch batch)
        //{
        //    try
        //    {
        //        copyHelper.SaveAll(connection, batch.Records);
        //        return new QueryStatus(true);
        //    }
        //    catch (Exception ex)
        //    {
        //        Log.Error(String.Format("Failed to insert batch into TimescaleDB. Exception: {0}", ex.ToString()));
        //        return new QueryStatus(false, 0, ex, ex.ToString());
        //    }
        //}

        async Task<QueryStatus> IDatabase.WriteBatch(Batch batch)
        {
            try
            {
                await copyHelper.SaveAllAsync(connection, batch.Records);
                return new QueryStatus(true);
            }
            catch (Exception ex)
            {
                Log.Error(String.Format("Failed to insert batch into TimescaleDB. Exception: {0}", ex.ToString()));
                return new QueryStatus(false, 0, ex, ex.ToString());
            }
        }

    }
}
    

